import {
  Color,
  EquirectangularReflectionMapping,
  Fog,
  Group,
  MeshStandardMaterial,
  PerspectiveCamera,
  RepeatWrapping,
  Scene,
  TextureLoader,
  WebGLRenderer,
} from "three";
import { GLTFLoader } from "three/addons/loaders/GLTFLoader.js";
import { OrbitControls } from "three/addons/controls/OrbitControls.js";
import { RGBELoader } from "three/addons/loaders/RGBELoader.js";
import { useEffect, useRef } from "react";
import { ThreeObj } from "@/typing/three.tsx";
import useModelStore from "@/store/useModelStore.ts";
import { textures } from "@/constant/textures.ts";
import { models } from "@/constant/models.ts";

const loadModel = (
  modelMeta: {
    name: string;
    path: string;
    scale: number;
  },
  onProgress?: (xhr: ProgressEvent) => void,
): Promise<Group> =>
  new Promise((resolve, reject) => {
    new GLTFLoader().load(
      modelMeta.path,
      (gltf) => {
        console.log(gltf);
        const model = gltf.scene;
        const scale = modelMeta.scale;
        model.scale.set(scale, scale, scale);
        model.position.set(0, 0, 0);
        resolve(model);
      },
      onProgress,
      (error) => {
        reject(error);
      },
    );
  });

const initTexture = (
  model: Group,
  activeModel: number,
  activeTextures: number[],
  activeArea: number,
) =>
  new Promise((resolve) => {
    model.traverse((_child) => {
      // eslint-disable-next-line @typescript-eslint/ban-ts-comment
      // @ts-expect-error
      if (_child.isMesh) {
        const ttr = models[activeModel].mesh.find(
          (el) => el.name === _child.name,
        );
        if (ttr) {
          const texture = new TextureLoader().load(
            textures[activeTextures[activeArea]].path,
          );
          // eslint-disable-next-line @typescript-eslint/ban-ts-comment
          // @ts-expect-error
          _child.material = new MeshStandardMaterial({
            map: texture,
          });
          // renderer.render(scene, camera);
        } else {
          // eslint-disable-next-line @typescript-eslint/ban-ts-comment
          // @ts-expect-error
          _child.material = new MeshStandardMaterial({
            color: 0x999999,
          });
          // renderer.render(scene, camera);
        }
      }
      resolve("success");
    });
  });
const ThreeDesigner = () => {
  const wrapRef = useRef<HTMLDivElement>(null);
  const threeRef = useRef<ThreeObj | null>(null);
  const activeTextures = useModelStore((state) => state.activeTextures);
  const activeArea = useModelStore((state) => state.activeArea);
  const activeModel = useModelStore((state) => state.activeModel);
  const setModelLoading = useModelStore((state) => state.setModelLoading);

  useEffect(() => {
    if (wrapRef.current) {
      // 场景
      const scene = new Scene();
      scene.background = new Color(0xf1f1f1);
      scene.environment = new RGBELoader().load("/venice_sunset_1k.hdr");
      scene.environment.mapping = EquirectangularReflectionMapping;
      scene.fog = new Fog(0x333333, 10, 15);

      // 渲染器
      const renderer = new WebGLRenderer();
      renderer.setSize(window.innerWidth, window.innerHeight);

      // 摄像机
      const camera = new PerspectiveCamera(
        75,
        window.innerWidth / window.innerHeight,
        0.1,
        1000,
      );
      camera.position.z = 600;

      // const mesh = new BoxGeometry();

      // 添加控制
      const controls = new OrbitControls(camera, renderer.domElement);
      controls.addEventListener("change", () => {
        renderer.render(scene, camera);
      });
      controls.minDistance = 2;
      controls.maxDistance = 10;
      controls.target.set(0, 0, -0.2);
      controls.update();

      // 添加光照
      // const ambientLight = new AmbientLight(0xffffff, 1); //环境光的颜色以及强弱
      // const pointLight = new PointLight(0xffffff, 1);
      // pointLight.position.set(0, 0, 1000);
      // scene.add(ambientLight);
      // scene.add(pointLight);

      loadModel(models[activeModel], (xhr) => {
        setModelLoading((xhr.loaded / xhr.total) * 100);
        console.log((xhr.loaded / xhr.total) * 100 + "% loaded");
      }).then((model) => {
        scene.add(model);
        renderer.render(scene, camera);
        initTexture(model, activeModel, activeTextures, activeArea).then(() => {
          setTimeout(() => {
            renderer.render(scene, camera);
          }, 500);
        });
      });

      wrapRef.current.appendChild(renderer.domElement);
      renderer.render(scene, camera);

      threeRef.current = {
        scene,
        renderer,
        camera,
      };
    }

    return () => {
      // 销毁
      threeRef.current?.renderer.domElement.remove();
      threeRef.current?.camera.remove();
      threeRef.current?.scene.remove();
      threeRef.current = null;
    };
  }, [activeModel]);

  /* activeTexture 变化时,切换texture */
  useEffect(() => {
    if (!threeRef.current) return;
    const { scene, renderer, camera } = threeRef.current;
    const texture = textures[activeTextures[activeArea]];
    const area = models[activeModel].mesh[activeArea];
    scene.traverse((child) => {
      if (child.type === "Mesh") {
        if (child.name === area.name) {
          console.log(area.name);
          new TextureLoader().load(texture.path, (texture) => {
            texture.wrapT = RepeatWrapping;
            texture.wrapS = RepeatWrapping;
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-expect-error
            child.material = new MeshStandardMaterial({
              map: texture,
            });
            renderer.render(scene, camera);
          });
        }
      }
    });
  }, [activeTextures, activeArea, activeModel]);
  return <div ref={wrapRef} className={"h-full w-full"}></div>;
};

export default ThreeDesigner;
